from random import choices


class Inject_Device:
    @staticmethod
    def _generate_device(
        app_version: str,
        device_model: str,
        system_version: str,
        token_type: str,
        token: str = "",
        lang_code: str = "fa",
        is_multi_account: bool = False,
    ) -> dict:
        return {
            "app_version": app_version,
            "device_hash": "".join(choices("0123456789", k=26)),
            "device_model": device_model,
            "is_multi_account": is_multi_account,
            "lang_code": lang_code,
            "system_version": system_version,
            "token": token,
            "token_type": token_type,
        }
    DeviceAndroid = _generate_device.__func__(
        app_version="MA_3.0.7",
        device_model="rulog",
        system_version="SDK 28",
        token_type="Firebase",
    )
    DeviceWeb = _generate_device.__func__(
        app_version="WB_4.3.3",
        device_model="rulog",
        system_version="Windows 11",
        token_type="Web",
    )
    AndroidRegister = _generate_device.__func__(
        app_version="MA_3.0.7",
        device_model="rulog",
        system_version="SDK 28",
        token_type="Firebase",
    )
    WebRegister = _generate_device.__func__(
        app_version="WB_4.4.20",
        device_model="chrom",
        system_version="Windows 10",
        token_type="Web",
    )


class AuthError(Exception):
    ...


class TypeMethodError(Exception):
    ...


class TypeAnti(Exception):
    ...


class ErrorServer(Exception):
    ...


class ErrorMethod(Exception):
    ...


class ErrorPrivatyKey(Exception):
    ...


class NOT_REGISTERED(Exception):
    ...


class TOO_REQUESTS(Exception):
    ...


from base64 import b64decode
from json import dump, loads
from re import findall
from .platform import Parts_Platform
from .Device import Inject_Device
from .Encoder import encoderjson
from .Error import ErrorMethod
from .PostData import Perager_postishon
from .RSA_ENC import deleteRSAset

from base64 import b64decode
from json import loads, dump
from re import findall

class Client:
    def __init__(self,password = None):
        self.password = password
    def register(self, platform: str, auths: str, keys: str) -> bool:
        if str(platform) in ("android",):
            keyAccount, Sh_account, Auth = deleteRSAset(keys), "".join(findall(r"\w{32}", auths)), encoderjson.changeAuthType("".join(findall(r"\w{32}", auths)))
            Keys = f"-----BEGIN RSA PRIVATE KEY-----\n{keyAccount}\n-----END RSA PRIVATE KEY-----"
        else:
            keyAccount, Sh_account, Auth = keys, "".join(findall(r"\w{32}", auths)), encoderjson.changeAuthType("".join(findall(r"\w{32}", auths)))
            Keys = loads(b64decode(keyAccount).decode('utf-8'))['d']
        method = Perager_postishon(
            plat=platform,
            OrginalAuth=Sh_account,
            auth=Auth,
            keyAccount=Keys
        )
        cli = Parts_Platform(platform).platform
        if platform == "web":
            return method.methodsRubika(
                'json',
                methode='registerDevice',
                indata=Inject_Device.WebRegister,
                wn=cli
            )
        else:
            return method.methodsRubika(
                'json',
                methode='registerDevice',
                indata=Inject_Device.AndroidRegister,
                wn=cli
            )



    def sendCode(self,platforms, numberphone: str, send_type: bool = False, password=None):
        """
        send_type: False -> 'SMS', True -> 'Internal'
        """
        cli = Parts_Platform(platforms).platform
        method = Perager_postishon()
        send_type_value = "Internal" if send_type else "SMS"

        print(numberphone)
        return method.methodsRubika(
            "login",
            methode="sendCode",
            indata={
                "phone_number": numberphone,
                "send_type": send_type_value,
                "pass_key": self.password,
            },
            wn=cli,
        )


    def signIn(self,platforms, numberphone: str, codehash, phone_code, save=None):
        publicKey, privateKey = encoderjson.rsaKeyGenerate()
        method = Perager_postishon()
        cli = Parts_Platform(platforms).platform
        if not (platforms and numberphone and codehash and phone_code):
            raise ErrorMethod("Enter the complete values ​​into the method")
        GetDataSignIn = method.methodsRubika(
            "login",
            methode="signIn",
            indata={
                "phone_number": numberphone,
                "phone_code_hash": codehash,
                "phone_code": phone_code,
                "public_key": publicKey,
                "private_key": privateKey,
            },
            wn=cli,
        )
        status = GetDataSignIn.get("data", {}).get("status")
        if status == "OK":
            data_account = dict(
                Auth=encoderjson.decryptRsaOaep(privateKey, GetDataSignIn.get("data").get("auth")),
                Key=privateKey,
            )
            if save is not None:
                with open(f"{save}.json", "a+") as f:
                    dump(data_account, f)
            return data_account

        if status == "CodeIsInvalid":
            raise ErrorMethod("Invalid Rubika login code")
        raise ErrorMethod(f"Unexpected signIn status: {status}")


import asyncio

import httpx
import nest_asyncio

from .Error import ErrorServer
from .GetDataMethod import GetDataMethod

list_servers, reoeatchoice_server, repeat = [], 0, 0

nest_asyncio.apply()


def Server_Rubika():
    while 1:
        try:
            for server_rubika in range(1):

                async def GETservers(server):
                    Retry = httpx.AsyncHTTPTransport(retries=5)
                    async with httpx.AsyncClient(
                        transport=Retry, http2=True, timeout=1.0
                    ) as client:
                        response = await client.get(url=server)
                        return response.json()

                for added in range(1):
                    servers = (
                        asyncio.run(GETservers("https://getdcmess.iranlms.ir/"))
                        .get("data")
                        .get("API")
                        .values()
                    )
                    list_servers.extend(servers)
                    list_servers.pop(1)
            break
        except httpx.ConnectError:
            raise ErrorServer("Please check your is not internet")
        except:
            continue


def defaultapi():
    global list_servers, reoeatchoice_server, repeat
    while 1:
        try:
            for tekrar in range(1):
                if len(list_servers) == 0:
                    reoeatchoice_server = 0
                    req = GetDataMethod(target=Server_Rubika, args=()).show()
                    if req:
                        continue
                elif len(list_servers) == 52:
                    if reoeatchoice_server < 52:
                        reoeatchoice_server += 1
                        repeat = reoeatchoice_server - 1
                        return list_servers[repeat]
                    elif reoeatchoice_server == 52:
                        list_servers, reoeatchoice_server, repeat = [], 0, 0
                        continue
        except:
            continue



from threading import Thread


class GetDataMethod(Thread):
    def __init__(self, target = None, args = ()):
        super().__init__(target = target, args = args)
        self._getdata = None

    def run(self):
        if self._target is not None:
            self._getdata = self._target(*self._args)

    def show(self):
        super().start()
        super().join()
        return self._getdata



import base64
import io
import json
import random
import re
import secrets
import string
from base64 import b64decode
from json import JSONDecoder, loads

from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import pkcs1_15
from Crypto.Util.Padding import pad, unpad
from PIL import Image

from .Error import AuthError


class encoderjson:
    def __init__(self, auth: str, private_key: str = None):
        self.auth = auth
        self.key = bytearray(self.createSecretPassphrase(auth), "UTF-8")
        self.iv = bytearray.fromhex("0" * 32)
        if private_key:
            self.keypair = RSA.import_key(private_key.encode("utf-8"))

    def changeAuthType(auth_dec):
        n = ""
        lowercase = "abcdefghijklmnopqrstuvwxyz"
        uppercase = lowercase.upper()
        digits = "0123456789"
        for s in auth_dec:
            if s in lowercase:
                n += chr(((32 - (ord(s) - 97)) % 26) + 97)
            elif s in uppercase:
                n += chr(((29 - (ord(s) - 65)) % 26) + 65)
            elif s in digits:
                n += chr(((13 - (ord(s) - 48)) % 10) + 48)
            else:
                n += s
        return n

    def decodeAuthType(auth_enc):
        n = ""
        lowercase = "abcdefghijklmnopqrstuvwxyz"
        uppercase = lowercase.upper()
        digits = "0123456789"
        for s in auth_enc:
            if s in lowercase:
                n += chr(((32 - (ord(s) - 97)) % 26) + 97)
            elif s in uppercase:
                n += chr(((29 - (ord(s) - 65)) % 26) + 65)
            elif s in digits:
                n += chr(((13 - (ord(s) - 48)) % 10) + 48)
            else:
                n += s
        return n

    def createSecretPassphrase(self, e):
        t, i = e[0:8], e[8:16]
        n = e[16:24] + t + e[24:32] + i

        for s in range(len(n)):
            e = n[s]
            if e >= "0" and e <= "9":
                t = chr(((ord(e) - ord("0") + 5) % 10) + ord("0"))
                n = n[:s] + t + n[s + 1 :]
            else:
                t = chr(((ord(e) - ord("a") + 9) % 26) + ord("a"))
                n = n[:s] + t + n[s + 1 :]
        return n

    def encrypt(self, text):
        try:
            encode_data = base64.b64encode(
                AES.new(self.key, AES.MODE_CBC, self.iv).encrypt(
                    pad(text.encode("UTF-8"), AES.block_size)
                )
            ).decode("UTF-8")
            return encode_data
        except:
            raise

    def decrypt(self, text):
        try:
            decode_data = unpad(
                AES.new(self.key, AES.MODE_CBC, self.iv).decrypt(
                    base64.urlsafe_b64decode(text.encode("UTF-8"))
                ),
                AES.block_size,
            ).decode("UTF-8")
            return decode_data
        except ValueError:
            raise AuthError(
                "Check your auth This auth is not the key to decrypt and encrypt data"
            )

    def makeSignFromData(self, data_enc: str):
        sha_data = SHA256.new(data_enc.encode("utf-8"))
        signature = pkcs1_15.new(self.keypair).sign(sha_data)
        return base64.b64encode(signature).decode("utf-8")

    def decryptRsaOaep(private: str, data_enc: str):
        keyPair = RSA.import_key(private.encode("utf-8"))
        return (
            PKCS1_OAEP.new(keyPair).decrypt(base64.b64decode(data_enc)).decode("utf-8")
        )

    def encryptRsaOaep(private: str, data_enc: str):
        keyPair = RSA.import_key(private.encode("utf-8"))
        return (
            PKCS1_OAEP.new(keyPair).encrypt(base64.b64encode(data_enc)).decode("utf-8")
        )

    def rsaKeyGenerate():
        keyPair = RSA.generate(1024)
        public = encoderjson.changeAuthType(
            base64.b64encode(keyPair.publickey().export_key()).decode("utf-8")
        )
        private = keyPair.export_key().decode("utf-8")
        return [public, private]


def getThumbInline(image_bytes: bytes):
    im = Image.open(io.BytesIO(image_bytes))
    width, height = im.size
    if height > width:
        new_height = 40
        new_width = round(new_height * width / height)
    else:
        new_width = 40
        new_height = round(new_width * height / width)
    im = im.resize((new_width, new_height), Image.ANTIALIAS)
    changed_image = io.BytesIO()
    im.save(changed_image, format="PNG")
    changed_image = changed_image.getvalue()
    return base64.b64encode(changed_image)



from setuptools import setup, find_packages

try:
    with open('README.md', encoding='utf-8') as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = ''

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='rulog',
    version='1.0.2',
    description=(
    "Rubika: A Python library for interacting with the Rubika Login APi. "
),

    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Mahdi Ahmadi',
    author_email='mahdiahmadi.1208@gmail.com',
    maintainer='Mahdi Ahmadi',
    maintainer_email='mahdiahmadi.1208@gmail.com',
    url='https://github.com/Mahdy-Ahmadi/rulog',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Communications :: Chat',
        'Topic :: Software Development :: Libraries',
        'Natural Language :: Persian',
    ],
    python_requires='>=3.6',
    install_requires=[
        "Pillow",
        'pycryptodome',
        'aiohttp',
        'httpx',
    ],
    keywords="rubika bot api library chat messaging rubpy pyrubi rubigram rubika_bot rubika_api fast_rub",
    project_urls={
        "Bug Tracker": "https://t.me/Bprogrammer",
        "Documentation": "https://github.com/Mahdy-Ahmadi/rulog/",
        "Source Code": "https://github.com/Mahdy-Ahmadi/rulog/tree/main/rulog",
    },
    license="MIT",
    zip_safe=False
)


import httpx
from json import dumps, loads, JSONDecodeError
from random import choices
from .Encoder import encoderjson
from .ErrorRubika import ErrorRubika
from .GetDataMethod import GetDataMethod
from .GtM import defaultapi


def SendRequests(plat: str, js: dict, OrginalAuth: str = None, auth: str = None, key: str = None, api_version: str = "6"):
    enc = encoderjson(auth, key)
    Enc = encoderjson(OrginalAuth, key)
    servers = GetDataMethod(target=defaultapi, args=()).show()
    payload = {
        "api_version": api_version,
        "auth": OrginalAuth if plat == "web" else auth,
        "data_enc": (enc if plat == "web" else Enc).encrypt(dumps(js)),
        "sign": enc.makeSignFromData((enc if plat == "web" else Enc).encrypt(dumps(js))),
    }
    headers = {"Content-Type": "application/json; charset=utf-8"}
    if plat == "web":headers["Referer"] = "https://web.rubika.ir/"
    with httpx.Client(http2=False, timeout=20.0) as client:
        try:
            response = client.post(servers, json=payload, headers=headers)
            response.raise_for_status()
            return response.text
        except httpx.HTTPError as e:raise RuntimeError(f"HTTP Error: {e}")
def logger_andector(auths: str, js: dict):
    servers = defaultapi()
    enc = encoderjson(auth=auths)
    payload = {
        "api_version": "6",
        "tmp_session": auths,
        "data_enc": enc.encrypt(dumps(js)),
    }
    with httpx.Client(http2=False, timeout=20.0) as client:
        response = client.post(servers, json=payload)
        response.raise_for_status()
        return response.text

class Perager_postishon:
    def __init__(self, plat: str = None, OrginalAuth: str = None, auth: str = None, keyAccount: str = None):
        self.Plat = plat
        self.Auth = auth
        self.OrginalAuth = OrginalAuth
        self.keyAccount = keyAccount
        self.enc = None
        if keyAccount:
            self.enc = encoderjson(auth if plat == "web" else OrginalAuth, keyAccount)

    def methodsRubika(self, types: str, methode: str, indata: dict, wn: dict = None, downloads: list = None, server: str = None, podata: bytes = None, header: dict = None):
        inData = {"method": methode, "input": indata, "client": wn}

        try:
            if types == "json":
                response_text = SendRequests(
                    plat=self.Plat,
                    js=inData,
                    OrginalAuth=self.OrginalAuth,
                    auth=self.Auth,
                    key=self.keyAccount,
                )
                data_enc = loads(response_text).get("data_enc")
                sendJS = loads(self.enc.decrypt(data_enc))

                if sendJS.get("status") != "OK":
                    err = ErrorRubika(sendJS)
                    if err.Error in ["re", "ra"]:
                        return err.state
                return sendJS
            elif types == "login":
                authrnd = encoderjson.changeAuthType("".join(choices("abcdefghijklmnopqrstuvwxyz", k=32)))
                self.enc = encoderjson(auth=authrnd)
                response_text = logger_andector(authrnd, inData)
                data_enc = loads(response_text).get("data_enc")
                sendLOGIN = loads(self.enc.decrypt(data_enc))

                if sendLOGIN.get("status") != "OK":
                    err = ErrorRubika(sendLOGIN)
                    if err.Error in ["re", "ra"]:
                        return err.state
                return sendLOGIN
        except JSONDecodeError:
            raise RuntimeError("Invalid JSON response.")
        except httpx.HTTPError as e:
            raise RuntimeError(f"HTTPX error: {e}")



class Parts_Platform:
    _platforms = {
        "android": {
            "app_name": "Main",
            "app_version": "3.5.7",
            "lang_code": "fa",
            "package": "app.rbmain.a",
            "temp_code": "31",
            "platform": "Android",
        },
        "web": {
            "app_name": "Main",
            "app_version": "4.4.6",
            "lang_code": "fa",
            "package": "web.rubika.ir",
            "platform": "Web",
        },
        "pwa": {
            "app_name": "Main",
            "app_version": "2.1.4",
            "lang_code": "fa",
            "package": "m.rubika.ir",
            "platform": "PWA",
        },
    }

    def __init__(self, platform: str):
        platform = platform.lower()
        self.platform = self._platforms.get(platform, self._platforms["pwa"]).copy()